@@ -1,10 +1,11 @@
-# Initialize logging
-import logging
 import os
 
 from .door import Door
 from .neighborhood import Neighborhood
 from .param import Param
 
-logging.basicConfig(filename=f"{os.getcwd()}/porchlight.log")
-loggers = logging.getLogger(__name__)
+
+# Initialize logging
+import logging
+
+logging.getLogger(__name__).addHandler(logging.NullHandler())
