@@ -1,5 +1,5 @@
 import porchlight
-from porchlight import Neighborhood, Door, Param
+from porchlight import Neighborhood, Door, Param, door
 
 import unittest
 from unittest import TestCase
@@ -13,13 +13,26 @@ logging.basicConfig(filename=f"{os.getcwd()}/porchlight_unittest.log")
 
 class TestNeighborhood(TestCase):
     def test___init___(self):
-        # Initialization should not do anything beyond defining the empty
+        # Plain initialization should not do anything beyond defining the empty
         # Neighborhood._doors and Neighborhood._params attrs
         neighborhood = Neighborhood()
 
         self.assertEqual(neighborhood._doors, {})
         self.assertEqual(neighborhood._params, {})
 
+    def test___init___with_list(self):
+        def test1(x):
+            pass
+
+        @door.Door
+        def test2(y: int) -> int:
+            z = y + 10
+            return z
+
+        neighborhood = Neighborhood([test1, test2])
+
+        self.assertEqual(list(neighborhood.params.keys()), ["x", "y", "z"])
+
     def test___repr__(self):
         neighborhood = Neighborhood()
         expected = r"Neighborhood(doors={}, params={}, call_order=[])"
@@ -32,17 +45,19 @@ class TestNeighborhood(TestCase):
 
         neighborhood.add_door(test1)
 
-        expected = (
-            "Neighborhood(doors={'test1': Door(name=test1, "
-            "base_function=<function test1 at 0x7f247d8c48b0>, "
-            "arguments={'x': <class 'int'>}, return_vals=[['y']])}, "
-            "params={'x': Param(name=x, value=<porchlight.param."
-            "Empty object at 0x7f247e1bd060>, constant=False, type=<class "
-            "'porchlight.param.Empty'>), 'y': Param(name="
-            "y, value=<porchlight.param.Empty object at 0x7f247d8c9660>, "
-            "constant=False, type=<class 'porchlight.param "
-            ".Empty'>)}, call_order=['test1'])"
-        )
+        # Keeping the below as an example of what the string looks like. Update
+        # this to match the actual implementation as changes occur below.
+        # expected = (
+        #     Neighborhood(doors={'test1': Door(name=test1,
+        #     base_function=<function TestNeighborhood.test___repr__.<lo
+        #     cals>.test1 at 0x7fdcf1333640>, arguments={'x': <class 'int'>},
+        #     return_vals=[['y']])}, params={'x': Para m(name=x,
+        #     value=<porchlight.param.Empty object at 0x7fdcf127f1f0>,
+        #     constant=False, type=<class 'porchlig ht.param.Empty'>), 'y':
+        #     Param(name=y, value=<porchlight.param.Empty object at
+        #     0x7fdcf127f1f0>, constant= False, type=<class
+        #     'porchlight.param.Empty'>)}, call_order=['test1'])
+        # )
 
         params = neighborhood.params
 
@@ -51,7 +66,6 @@ class TestNeighborhood(TestCase):
             f"params={params}, call_order=['test1'])"
         )
 
-        print(repr(neighborhood))
         self.assertEqual(repr(neighborhood), expected)
 
     def test_add_function(self):
@@ -263,6 +277,10 @@ class TestNeighborhood(TestCase):
             neighborhood._params["rettest"].value, expected_param.value
         )
 
+        # Test running multiple steps
+        n = 5
+        neighborhood.run_steps(n)
+
         # Make one of the parameters constant.
         neighborhood.set_param("rettest", 1, constant=True)
 
@@ -453,7 +471,9 @@ class TestNeighborhood(TestCase):
         neighborhood = Neighborhood()
 
         neighborhood.add_function(fxn_one)
-        neighborhood.add_function(fxn_two)
+
+        with self.assertWarns(door.DoorWarning):
+            neighborhood.add_function(fxn_two)
 
         # Provide the required first arg, x
         neighborhood.set_param("x", 0)
@@ -563,11 +583,11 @@ class TestNeighborhood(TestCase):
 
         # Add two dynamic doors using a single generator function.
         def doublegen_test(
-            x: int,
+            x: float,
         ) -> typing.Tuple[porchlight.Door, porchlight.Door]:
             @porchlight.Door
             def test1(y: float) -> float:
-                z = y ** x + 1
+                z = y**x + 1
                 return z
 
             @porchlight.Door
@@ -579,8 +599,8 @@ class TestNeighborhood(TestCase):
 
         neighborhood = Neighborhood()
         neighborhood.add_function(doublegen_test, dynamic_door=True)
-        neighborhood.add_param("x", 0)
-        neighborhood.add_param("y", 2)
+        neighborhood.add_param("x", 0.0)
+        neighborhood.add_param("y", 2.0)
 
         expected_doors = ["doublegen_test", "test1", "test2"]
         expected_params = ["x", "test1", "test2", "y", "z"]
@@ -590,16 +610,16 @@ class TestNeighborhood(TestCase):
         neighborhood.run_step()
 
         self.assertEqual(list(neighborhood.params.keys()), expected_params)
-        self.assertEqual(neighborhood.params["x"].value, 4)
-        self.assertEqual(neighborhood.params["y"].value, 2)
-        self.assertEqual(neighborhood.params["z"].value, 2)
+        self.assertAlmostEqual(neighborhood.params["x"].value, 4.0)
+        self.assertAlmostEqual(neighborhood.params["y"].value, 2.0)
+        self.assertAlmostEqual(neighborhood.params["z"].value, 2.0)
 
         neighborhood.run_step()
 
         self.assertEqual(list(neighborhood.params.keys()), expected_params)
-        self.assertEqual(neighborhood.params["x"].value, 34)
-        self.assertEqual(neighborhood.params["y"].value, 2)
-        self.assertEqual(neighborhood.params["z"].value, 17)
+        self.assertAlmostEqual(neighborhood.params["x"].value, 34.0)
+        self.assertAlmostEqual(neighborhood.params["y"].value, 2.0)
+        self.assertAlmostEqual(neighborhood.params["z"].value, 17.0)
 
     def test_bad_dynamic_door(self):
         @Door
@@ -631,6 +651,260 @@ class TestNeighborhood(TestCase):
         self.assertEqual(result["x"], Param("x", porchlight.param.Empty()))
         self.assertEqual(result["y"], Param("y", "15.5"))
 
+    def test_bad_dynamicdoor_return_values(self):
+        @porchlight.door.DynamicDoor
+        def test1():
+            def bad():
+                x = 1
+                return x
+
+            return bad
+
+        neighborhood = Neighborhood()
+
+        with self.assertRaises(porchlight.neighborhood.NeighborhoodError):
+            neighborhood.add_door(test1, dynamic_door=True)
+
+    def test_get_value(self):
+        def test1(x, y=1):
+            z = x + y
+            return z
+
+        neighborhood = Neighborhood()
+
+        neighborhood.add_function(test1)
+        neighborhood.set_param("x", 1)
+        neighborhood.run_step()
+
+        expected = {
+            "x": 1,
+            "y": 1,
+            "z": 2,
+        }
+
+        for pname, val in expected.items():
+            self.assertEqual(val, neighborhood.get_value(pname))
+
+    def test_initialization(self):
+        def inittest1():
+            x = 5
+            return x
+
+        def test1(x):
+            y = x**2
+            return y
+
+        neighborhood = Neighborhood([test1], initialization=inittest1)
+
+        # try to run a step
+        neighborhood.run_step()
+
+        # Check the parameters for expected values.
+        self.assertEqual(neighborhood.get_value("x"), 5)
+        self.assertEqual(neighborhood.get_value("y"), 25)
+
+        @door.BaseDoor
+        def inittest2(x):
+            z = 5 * x
+            return z
+
+        def test1(x):
+            y = x**2
+            return y
+
+        neighborhood = Neighborhood(
+            [test1], initialization=[inittest1, inittest2]
+        )
+
+        # try to run a step
+        neighborhood.run_step()
+
+        # Check the parameters for expected values.
+        self.assertEqual(neighborhood.get_value("x"), 5)
+        self.assertEqual(neighborhood.get_value("y"), 25)
+
+    def test_finalization(self):
+        def inittest1():
+            x = 5
+            return x
+
+        def test1(x):
+            y = x**2
+            return y
+
+        def fintest1(x, y):
+            x += 1
+            y *= 2
+            z = x + y
+            print("Testing finalization and values are:")
+            print(f"\t{x = }")
+            print(f"\t{y = }")
+            print(f"\t{z = }")
+
+            return x, y, z
+
+        # This tests returns that are not tuples
+        @door.BaseDoor
+        def fintest2(x):
+            x_string = f"x has value: {x}"
+            return x_string
+
+        neighborhood = Neighborhood(
+            [test1], initialization=inittest1, finalization=fintest1
+        )
+
+        # try to run a step
+        neighborhood.run_step()
+
+        neighborhood.finalize()
+
+        # Check the parameters for expected values.
+        self.assertEqual(neighborhood.get_value("x"), 6)
+        self.assertEqual(neighborhood.get_value("y"), 50)
+        self.assertEqual(neighborhood.get_value("z"), 56)
+
+        # Testing with a finalization list
+        neighborhood = Neighborhood(
+            [test1], initialization=inittest1, finalization=[fintest1, fintest2]
+        )
+
+        # try to run a step
+        neighborhood.run_step()
+
+        neighborhood.finalize()
+
+        # Check the parameters for expected values.
+        self.assertEqual(neighborhood.get_value("x"), 6)
+        self.assertEqual(neighborhood.get_value("y"), 50)
+        self.assertEqual(neighborhood.get_value("z"), 56)
+
+    def test_call(self):
+        @door.Door
+        def test1(x: int = 1) -> str:
+            status_string = "large" if x >= 10 else "small"
+
+            return status_string
+
+        @door.Door
+        def test2(x: int, y: int = 14) -> str:
+            test_sum = x + y
+            status_string = "large" if test_sum >= 10 else "small"
+
+            return status_string
+
+        # This example tests functions with no return value.
+        @door.Door
+        def test3(y, /):
+            pass
+
+        neighborhood = Neighborhood([test1, test2, test3])
+
+        # 'x' not required for test 1, and will be propogated to test2 when
+        # called subsequently.
+        neighborhood.call("test1")
+        self.assertEqual(neighborhood.get_value("status_string"), "small")
+
+        neighborhood.call("test2")
+        self.assertEqual(neighborhood.get_value("status_string"), "large")
+
+        neighborhood.call("test3")
+        self.assertEqual(neighborhood.get_value("status_string"), "large")
+        self.assertEqual(neighborhood.get_value("y"), 14)
+
+    def test_call_bad_door_name(self):
+        def test1():
+            pass
+
+        neighborhood = Neighborhood(test1)
+
+        call_tests = ("bing_bong", "Enclkjd dksjfh e", "####", "ch31 3 42")
+
+        for test_str in call_tests:
+            with self.assertRaises(porchlight.neighborhood.NeighborhoodError):
+                neighborhood.call(test_str)
+
+    def test_call_basedoor(self):
+        @door.BaseDoor
+        def test1():
+            pass
+
+        neighborhood = Neighborhood(test1)
+
+        neighborhood.call("test1")
+
+    def test_initialize_dynamicdoor(self):
+        # Since this is not implemented/supported, need to eensure it throws an
+        # error for now.
+        @door.DynamicDoor
+        def test_dynamicdoor() -> door.Door:
+            @door.Door
+            def test_door_gen():
+                return
+
+            return test_door_gen
+
+        neighborhood = Neighborhood(initialization=[test_dynamicdoor])
+
+        with self.assertRaises(NotImplementedError):
+            neighborhood.initialize()
+
+    def test_call_door_by_reference(self):
+        @door.Door
+        def test_1():
+            pass
+
+        @door.Door
+        def test_2():
+            return
+
+        @door.Door
+        def test_3(x, y, z):
+            total = sum((x, y, z))
+            return total
+
+        neighborhood = Neighborhood((test_1, test_2, test_3))
+
+        init_params = {"x": 1, "y": 2, "z": 3.0}
+
+        for param, value in init_params.items():
+            neighborhood.add_param(param, value)
+
+        # Test calling these doors via reference
+        neighborhood.call(test_1)
+        neighborhood.call(test_2)
+        neighborhood.call(test_3)
+
+        cur_state = {
+            p: v.value
+            for p, v in neighborhood.params.items()
+            if p in init_params
+        }
+
+        for param, value in cur_state.items():
+            self.assertEqual(value, init_params[param])
+
+        # Pass a bad reference
+        with self.assertRaises(ValueError):
+
+            @door.Door
+            def bad_door_1():
+                pass
+
+            neighborhood.call(bad_door_1)
+
+        tests = (
+            1,
+            1.0,
+            [],
+            [1],
+            (1,),
+            lambda x: x,
+        )
+
+        for test in tests:
+            with self.assertRaises(TypeError):
+                neighborhood.call(test)
+
 
 if __name__ == "__main__":
     import unittest
